/* Copyright (c) Coveo Solutions Inc. */

(function(window, document, $, Granite) {
    "use strict";

    var REINDEX_URL = Granite.HTTP.internalize("/etc/workflow/instances?model=/var/workflow/models/coveo-reindex&payloadType=JCR_PATH&comment=Workflow started from code&workflowTitle=Reindexing Content&payload=");
    var COVEO_REINDEX_TITLE = Granite.I18n.get("Coveo Reindex Content");
    var BUTTON_TEXT = Granite.I18n.get("Reindex");
    var CANCEL_TEXT = Granite.I18n.get("Cancel");
    var ERROR_TEXT = Granite.I18n.get("Error");

    function activatePagesAndItsReferences(config, collection, selections) {
        var ui = $(window).adaptTo("foundation-ui");
        ui.wait();

        // get pages
        var paths = selections.map(function(v) {
            var item = $(v);

            // allow to set user defined reference paths, split by comma
            var refPath = item.data("checkReferencesPath");
            if (!refPath) {
                // coral3 selection is acting on the masonry level
                refPath = item.children().data("checkReferencesPath");
            }

            if (refPath) {
                return refPath.split(",");
            }

            return item.data("foundationCollectionItemId");
        });

        if (!paths.length) return;

        // flatten the paths
        paths = removeDuplicatesInArray(paths);

        // get the references
        var referencePromise = $.ajax({
            url: Granite.URITemplate.expand(config.data.referenceSrc, {
                path: paths
            }),
            "type": "POST",
            cache: false,
            dataType: "json"
        }).fail(function(xhr) {
            var message = Granite.I18n.getVar($(xhr.responseText).find("#Message").html());
            ui.alert(ERROR_TEXT, message, "error");
        });

        // merge pages and references paths
        referencePromise.done(function(json) {
            // merge all paths
            if (json.assets.length) {
                for (var i = 0; i < json.assets.length; i++ ){
                    var reference = json.assets[i];
                    paths.push(reference.path);
                }
                // flatten the paths
                paths = removeDuplicatesInArray(paths);
            }

            // reindex the page and its children
            $.ajax({
                url: REINDEX_URL + paths[paths.length - 1],
                type: "POST",
                data: {
                    _charset_: "utf-8"
                }
            }).always(function() {
                ui.clearWait();
            }).done(function() {
                var api = $(collection).adaptTo("foundation-collection");

                if (api && "reload" in api) {
                    api.reload();
                    ui.notify(null, getSuccessMessage(selections));
                    return;
                }

                var contentApi = $(".foundation-content").adaptTo("foundation-content");
                if (contentApi) {
                    contentApi.refresh();
                }

                ui.notify(null, getSuccessMessage(selections));
            }).fail(function(xhr) {
                var title = Granite.I18n.get("Error");
                var message = Granite.I18n.getVar($(xhr.responseText).find("#Message").html());

            if (message.includes('workflow model')) {
                var coveoModelName = '\"coveo-reindex\"';
                var coveoLinkTitle = 'Create a Workflow Model';
                var coveoMessage = `The ${coveoModelName} workflow model was not found.<br>Learn how to create it:`;
                var documentationLink = '\"https://docs.coveo.com/en/l8dc0578\"';
                message = `${coveoMessage} <i><a href=${documentationLink} target="_blank" rel="noopener noreferrer">${coveoLinkTitle}</a></i>.`;
            }

                ui.alert(title, message, "error");
            });
        });
    }

    function createEl(name) {
        return $(document.createElement(name));
    }

    function getSuccessMessage(selections) {
        var successMessage = Granite.I18n.get("The {0} pages and their children have been reindexed", selections.length);
        if (selections.length === 1) {
            successMessage = Granite.I18n.get("The page and its children have been reindexed");
        }
        return successMessage;
    }

    function removeDuplicatesInArray(anyArray) {
        return anyArray.sort().filter(function(item, pos, ary) {
            return !pos || item != ary[pos - 1];
        });
    }

    $(window).adaptTo("foundation-registry").register("foundation.collection.action.action", {
        name: "cq.wcm.coveoindex",
        handler: function(name, el, config, collection, selections) {
            var message = createEl("div");

            var intro = createEl("p").appendTo(message);
            if (selections.length === 1) {
                intro.text(Granite.I18n.get("The page and its children will be reindexed."));
            } else {
                intro.text(Granite.I18n.get("The {0} pages and their children will be reindexed.", selections.length));
            }

            var ui = $(window).adaptTo("foundation-ui");
            ui.prompt(COVEO_REINDEX_TITLE, message.html(), "notice", [{
                text: CANCEL_TEXT
            }, {
                text: BUTTON_TEXT,
                primary: true,
                handler: function() {
                    activatePagesAndItsReferences(config, collection, selections);
                }
            }]);
        }
    });
})(window, document, Granite.$, Granite);
